/* *************************************************************************
 
                               IT Mill Toolkit 

               Development of Browser User Interfaces Made Easy

                    Copyright (C) 2000-2006 IT Mill Ltd
                     
   *************************************************************************

   This product is distributed under commercial license that can be found
   from the product package on license.pdf. Use of this product might 
   require purchasing a commercial license from IT Mill Ltd. For guidelines 
   on usage, see licensing-guidelines.html

   *************************************************************************
   
   For more information, contact:
   
   IT Mill Ltd                           phone: +358 2 4802 7180
   Ruukinkatu 2-4                        fax:   +358 2 4802 7181
   20540, Turku                          email:  info@itmill.com
   Finland                               company www: www.itmill.com
   
   Primary source for information and releases: www.itmill.com

   ********************************************************************** */

package com.itmill.toolkit.terminal;

import java.net.URL;

import com.itmill.toolkit.service.FileTypeResolver;

/** External resource implements source for resources fetched from 
 * location specified by URL:s. The resources are fetched directly by the
 * client terminal and are not fetched trough the terminal adapter.
 * 
 * @author IT Mill Ltd.
 * @version @VERSION@
 * @since 3.0
 */
public class ExternalResource implements Resource {

	/** Url of the download */
	private String sourceURL = null;


	/** Create new download component for downloading directly from given URL. 
	 * */
	public ExternalResource(URL sourceURL) {
		if (sourceURL == null) 
			throw new RuntimeException("Source must be non-null");
			
		this.sourceURL = sourceURL.toString();
	}

	/** Create new download component for downloading directly from given URL. 
	 * */
	public ExternalResource(String sourceURL) {
		if (sourceURL == null) 
			throw new RuntimeException("Source must be non-null");
			
		this.sourceURL = sourceURL.toString();
	}

	/** Get the URL of the external resource */
	public String getURL() {
		return sourceURL;
	}
	
	public String getMIMEType() {
		return FileTypeResolver.getMIMEType(getURL().toString());
	}

}
